"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const IORedis = require("ioredis");
class RedisLayer {
    constructor(options) {
        this.redis = new IORedis(options.uri);
        if (options.ttl !== undefined && options.ttl < 0) {
            throw new Error('ttl should be ≥ 0');
        }
        this._options = Object.assign({ ttl: Infinity, keyPrefix: 'cacheloader', keyFn: key => String(key), serializer(value) {
                return JSON.stringify(value);
            },
            deserializer(data) {
                try {
                    return JSON.parse(data);
                }
                catch (e) {
                    return e;
                }
            } }, options);
        this._parseKey = this._parseKey.bind(this);
    }
    async batchGet(keys) {
        const results = await this.redis.mget(...keys.map(this._parseKey));
        return results.map((value, index) => {
            if (!value) {
                return new Error(`${index}: No cache.`);
            }
            return this._options.deserializer(value);
        });
    }
    batchSet(map) {
        if (this._options.ttl === Infinity) {
            const parsedMap = new Map();
            map.forEach((value, key) => {
                parsedMap.set(this._parseKey(key), this._options.serializer(value));
            });
            this.redis.mset(parsedMap).catch(e => {
                console.error('Redis write failed.', e.message);
            });
        }
        else {
            map.forEach((value, key) => {
                this.redis.psetex(this._parseKey(key), this._options.ttl, this._options.serializer(value)).catch(e => {
                    console.error('Redis write failed.', e.message);
                });
            });
        }
    }
    _parseKey(key) {
        return `${this._options.keyPrefix}.${this._options.keyFn(key)}`;
    }
}
exports.default = RedisLayer;
//# sourceMappingURL=RedisLayer.js.map