"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Debug = require("debug");
const debug = Debug('LayeredCacheLoader:TTLMap');
class TTLMap extends Map {
    constructor(entries) {
        super(entries);
        this._ttls = new Map();
    }
    setTTL(key, value, ttl) {
        super.set(key, value);
        this._clearTimer(key);
        if (Number.isSafeInteger(ttl)) {
            this._ttls.set(key, setTimeout(() => {
                debug(`TTL: ${key}`);
                this._clearTimer(key);
            }, ttl));
        }
        return this;
    }
    delete(key) {
        this._clearTimer(key);
        return super.delete(key);
    }
    clear() {
        this._clearAllTimers();
        return super.clear();
    }
    _clearTimer(key) {
        const timer = this._ttls.get(key);
        if (timer !== undefined) {
            this._ttls.delete(key);
            super.delete(key);
            clearTimeout(timer);
        }
    }
    _clearAllTimers() {
        this._ttls.forEach(value => clearTimeout(value));
        this._ttls.clear();
    }
}
exports.default = TTLMap;
//# sourceMappingURL=TTLMap.js.map